=begin

This file is part of Teatime.

Teatime is free software: you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation,
version 3 of the License.

Teatime is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
Teatime. If not, see http://www.gnu.org/licenses/.


Copyright 1998-2009 Detlef Reichl  detlef!reichl()gmx!org

=end

module Gtk
    class Dialog
        signal_new(:cup_changed, GLib::Signal::RUN_FIRST,
                   nil, GLib::Type['void'])
        def signal_do_cup_changed
        end
    end
end


module Teatime
    module Preferences
        class Dialog < Gtk::Dialog
            include GetText
            GetText.bindtextdomain 'teatime'

            def initialize conf
                super _('Teatime properties'), nil, nil, [Gtk::Stock::CLOSE, Gtk::Dialog::RESPONSE_CLOSE]
                @conf = conf
                self.set_default_size 600, -1
                self.set_icon PIXMAP_PATH + '/teatime.png'

                notebook = Gtk::Notebook.new
                self.vbox.pack_start notebook, false, false, 6


                # TIMES PAGE
                vbox =Gtk::VBox.new
                notebook.append_page vbox, Gtk::Label.new(_('drawing time'))
                @times_vbox =Gtk::VBox.new
                vbox.pack_start @times_vbox, false, false, 0

                @time_labels = []
                @time_scales = []
                @teas = @conf['teas']
                @times = @conf['times']
                0.upto(@teas.length() - 1) { |i| add_tea i }
                hbox = Gtk::HBox.new false, 6
                vbox.pack_start hbox, false, false, 6
                button = Gtk::Button.new(_('Add tea'))
                hbox.pack_start button, false, false, 0
                button.signal_connect(:clicked)      { on_add_tea_activated }

                @tea_entry = Gtk::Entry.new
                @tea_entry.text = _('New tea')
                hbox.pack_start @tea_entry, true, true, 0
                @tea_entry.signal_connect(:activate) { on_add_tea_activated }


                # CUPS PAGE
                cups_vbox = Gtk::VBox.new false, 6
                label = Gtk::Label.new _('Choose your tea cup')
                cups_vbox.pack_start label, false, false, 12
                cups_hbox = Gtk::HBox.new true, 2
                cups_vbox.pack_start cups_hbox, false, false, 6
                notebook.append_page cups_vbox, Gtk::Label.new(_('tea cup'))

                button = nil
                @cup_buttons = []
                0.upto(2) do |i|
                    cup = Teatime::CUP_NAMES[i]
                    image = Gtk::Image.new PIXMAP_PATH + '/applet/' + cup + '_full_0.png'
                    button = Gtk::RadioButton.new button
                    @cup_buttons << button
                    button.signal_connect_after(:toggled) do
                        @cup_buttons.each do |btn|
                            if btn.active?
                                @conf['cup'] = @cup_buttons.index(btn)
                                signal_emit :cup_changed
                                break
                            end
                        end
                    end
                    button << image
                    cups_hbox.pack_start button, true, false, 6
                end
                @cup_buttons[@conf['cup']].active = true


                # ALARM PAGE
                alarm_vbox = Gtk::VBox.new false, 0
                notebook.append_page alarm_vbox, Gtk::Label.new(_('alarm'))

                button0 = Gtk::CheckButton.new _('Open window if tea is ready')
                button0.active = @conf['popup']
                button0.signal_connect(:toggled) { |button, event| @conf['popup'] = button.active? }
                alarm_vbox.pack_start button0, false, false, 6

                button1 = Gtk::CheckButton.new _('Play sound if tea is ready')
                button1.active = @conf['play_sound']
                button1.signal_connect(:toggled) { |button, event| @conf['play_sound'] = button.active? }
                alarm_vbox.pack_start button1, false, false, 6

                button2 = Gtk::FileChooserButton.new _('Sound to play'), Gtk::FileChooser::ACTION_OPEN
                filename = @conf['sound']
                button2.filename = filename unless filename == ''
                if filename == ''
                    folder = File.expand_path '~/'
                else
                    folder = File.dirname filename
                end
                button2.set_current_folder folder
                button2.signal_connect(:file_set) { |button| on_sound_file_selected button }
                alarm_vbox.pack_start button2, false, false, 2

                button3 = Gtk::CheckButton.new _('Show clock while drawing')
                button3.active = @conf['show_clock']
                button3.signal_connect(:toggled) { |button, event| @conf['show_clock'] = button.active? }
                alarm_vbox.pack_start button3, false, false, 12

                # The order of the following commands is ugly, but if we first set the page
                # and than show it, we get a switch_page signal with the old page.
                # It is explained with "historical reasons" in the Gtk+ docs...
                notebook.show_all
                notebook.page = @conf['pref_page']
                notebook.signal_connect(:switch_page) { |widget, pg, page| @conf['pref_page'] = page }
            end
            attr_reader :times

            
            def on_sound_file_selected button
                @conf['sound'] = button.filename
            end


            def on_add_tea_activated
                tea = @tea_entry.text.strip
                message = nil
                if @teas.include? tea
                    message = _("%s allready exist") % tea
                elsif tea == ''
                    message = _('Please insert a name')
                else
                    i = @teas.length
                    @teas << tea
                    @times << 210
                    @conf['teas'] = @teas
                    @conf['times'] = @times
                    add_tea i
                    @times_vbox.show_all
                    @tea_entry.text = _('New tea')
                    return
                end
                dialog = Gtk::MessageDialog.new nil,
                                                Gtk::Dialog::DESTROY_WITH_PARENT | Gtk::Dialog::MODAL,
                                                Gtk::MessageDialog::INFO,
                                                Gtk::MessageDialog::BUTTONS_CLOSE,
                                                message
                dialog.set_icon PIXMAP_PATH + '/teatime.png'
                dialog.run
                dialog.destroy
            end


            def add_tea i
                tea = @teas[i]
                time = @times[i]

                table = Gtk::Table.new 2, 3, false
                @times_vbox.pack_start table, false, false, 4

                # the label witht the tea name
                label = Gtk::Label.new(_(tea))
                table.attach_defaults label, 0, 1, 0, 1

                # the drawing time label
                time_label = Gtk::Label.new ""
                @time_labels << time_label
                table.attach time_label, 1, 2, 0, 1, Gtk::EXPAND | Gtk::FILL, Gtk::SHRINK, 0, 0

                # the drawing time slider
                adj_val = adjustment_from_drawingtime time
                adj = Gtk::Adjustment.new adj_val, 3.334, 304.9999, 0.1, 3.3333, 0.1
                time_scale = Gtk::HScale.new adj
                @time_scales << time_scale
                time_scale.draw_value = false
                time_scale.update_policy = Gtk::UPDATE_CONTINUOUS
                table.attach time_scale, 0, 2, 1, 2, Gtk::EXPAND | Gtk::FILL, Gtk::SHRINK, 0, 0
                time_scale.signal_connect(:value_changed) { on_time_scale_value_changed i }
                on_time_scale_value_changed i

                # the delete button
                image = Gtk::Image.new Gtk::Stock::DELETE, Gtk::IconSize::BUTTON
                button = Gtk::Button.new << image
                table.attach button, 2, 3, 0, 2, Gtk::SHRINK, Gtk::SHRINK, 0, 0
                button.signal_connect(:clicked) do
                    if @time_labels.length > 0
                        @time_labels.delete time_label
                        @time_scales.delete time_scale
                        table.destroy

                        @teas.delete tea
                        @times.delete time
                        @conf['teas'] = @teas
                        @conf['times'] = @times
                    end
                end
            end


            def adjustment_from_drawingtime time
                return time / 3.0            if time < 60
                return (time - 60) / 10 + 20 if time < 600
                return (time - 600) / 60 + 74
            end

            def on_time_scale_value_changed index
                scale = @time_scales[index]
                label = @time_labels[index]
                val = scale.adjustment.value

                sec = nil
                min = nil
                hour = nil

                if val < 20
                    sec = (val * 3).to_i
                elsif val < 74
                    val = (val - 20) * 10 + 60
                    min = (val / 60).to_i
                    sec = (val / 10).to_i * 10 % 60
                else
                    val = (val - 74) * 60 + 600
                    min = (val / 60).to_i
                    if min >= 60
                        hour = (min / 60).to_i
                        min  = min % 60
                    end
                end

                text = nil
                if hour
                    text = _("%i std %-2.2i min") % [hour, min]
                elsif min
                    text = _("%i min %-2.2i sec") % [min, sec]
                else
                    text = _("%i sec") % sec
                end
                label.text = text
                
                time = 0
                time += hour * 3600 if hour
                time += min * 60 if min
                time += sec if sec
                @times[index] = time
            end
        end


        class Conf
            def initialize applet
                @applet = applet
                @gconf_keys = {'cup' => [:integer, 0],                # the selected teacup
                               'last_tea' => [:integer, 0],           # the last used tea
                               'play_sound' => [:boolean, false],     # if a sound should be played after drawing
                               'popup' => [:boolean, true],           # if a popup is displayed after drawing
                               'pref_page' => [:integer, 0],          # the last page opend in the preferences
                               'show_clock' => [:boolean, true],      # show a clock while drawing
                               'sound' => [:string, ''],              # the sound, that shall be played after drawing
                               'teas' => [:string, 'Assam_Darjeeling_Fruit tea_Green tea_Herbal tea_Mint tea'],
                               'times' => [:string, '260_220_720_180_600_450']} # The drawing times
            end


            def [](key)
                unless @gconf_keys.has_key? key
                    printf "Warning: unknown key ", key
                    return nil
                end

                pair = @gconf_keys[key]

                begin
                    ret = case pair[0]
                        when :string  then @applet.gconf_get_string key
                        when :boolean then @applet.gconf_get_bool   key
                        when :integer then @applet.gconf_get_int    key
                    end
                rescue
                    if $log
                        $log.printf "fetching of the gconf key %s failed with\n%s\n", key, $!
                        $log.flush
                    end
                end

                ret = pair[1] if ret.nil?
                return ret.split("_") if key == 'teas'
                return ret unless key == 'times'

                ary_new = []
                ary_old = ret.split("_")
                ary_old.each { |element| ary_new << element.to_i }
                return ary_new
            end

            
            def[]=(key, val)
                unless @gconf_keys.has_key? key
                    printf "Warning: unknown key ", key
                    return
                end
                
                if key == 'teas' or key == 'times'
                    if key == 'teas'
                        val.each { |word| word.tr! "_", " " }
                    else
                        ary = []
                        val.each { |int| ary << int.to_s }
                        val = ary
                    end
                    begin
                        @applet.gconf_set_string key, val.join("_")
                    rescue
                        if $log
                            $log.printf "seting of the gconf key %s with value %s failed with\n%s\n", key, val.to_s, $!
                            $log.flush
                        end
                    end
                else
                    begin
                        case @gconf_keys[key][0]
                            when :string  then @applet.gconf_set_string key, val
                            when :boolean then @applet.gconf_set_bool   key, val
                            when :integer then @applet.gconf_set_int    key, val
                        else
                            raise RuntimeError
                        end
                    rescue
                        if $log
                            $log.printf "seting of the gconf key %s with value %s failed with\n%s\n", key, val.to_s, $!
                            $log.flush
                        end
                    end
                end
            end
        end
    end

    # This is only needed to get some strings into translations
    def lang_dummy
        _('Assam')
        _('Darjeeling')
        _('Fruit tea')
        _('Green tea')
        _('Herbal tea')
        _('Mint tea')
    end
end
